use std::sync::Arc;

use axum::{serve, Router};
use config::db::init_db;
use dotenvy::dotenv;
use tokio::net::TcpListener;
use crate::routes::user::controller::UserController;
use crate::routes::user::service::UserService;

mod base;
mod config;
mod routes;

#[tokio::main]
async fn main() {
    dotenv().ok();
    let pool = init_db();

    let user_service = UserService::new(pool);
    let user_controller = Arc::new(UserController::new(user_service));

    let app = Router::new()
        .merge(UserController::routes(user_controller.clone()));

    let listener = TcpListener::bind("0.0.0.0:3000").await.unwrap();
    serve(listener, app).await.unwrap();
}
