use std::sync::Arc;

use axum::extract::State;
use axum::http::StatusCode;
use axum::response::IntoResponse;
use axum::{Json, Router};
use axum::routing::get;
use crate::base::controller::BaseController;

use super::model::User;
use super::service::UserService;

pub struct UserController {
    base_controller: BaseController<User, UserService>,
}

impl UserController {
    pub fn new(service: UserService) -> Self {
        Self {
            base_controller: BaseController::new(service),
        }
    }

    pub fn routes(state: Arc<UserController>) -> Router {
        Router::new()
            .route("/users", get(Self::get_users))
            .with_state(state)
    }

    async fn get_users(
        State(controller): State<Arc<UserController>>,
    ) -> Result<impl IntoResponse, (StatusCode, Json<String>)> {
        controller.base_controller.get_all().await.map_err(|err| {
            (StatusCode::INTERNAL_SERVER_ERROR, Json(err))
        })    
    }
}
